import java.awt.*;
import java.awt.event.*;
import java.util.Random;

import javax.swing.*;

public class SearchGui implements ActionListener {

   static enum Alg {Linear, ImprovedLinear, Binary, Interpolation};

   public static void main(String[] args) {
      new SearchGui();
   }

   final JFrame frame;
   final JComboBox alg;
   final JTextField size, iterations;
   final JTextArea area;
   final JButton start;
   final JButton stop;
   final long seed;
   Thread thread = null;

   SearchGui() {
      seed = new Random().nextLong(); 
      frame = new JFrame("Search-Test");
      frame.setPreferredSize(new Dimension(600,400));
      JPanel panel = new JPanel(new BorderLayout());
      JPanel startStop = new JPanel(new GridLayout(1,2));
      start = new JButton("Start");
      start.addActionListener(this);
      stop = new JButton("Stop");
      stop.addActionListener(this);
      stop.setEnabled(false);
      frame.setContentPane(panel);
      area = new JLogArea();
      area.setText("Testing search algorithms\n");
      JScrollPane scroll = new JScrollPane(area);
      panel.add(scroll, BorderLayout.CENTER);
      JPanel config = new JPanel(new GridLayout(1,4));
      alg = new JComboBox(Alg.values());
      JButton clear = new JButton("Clear");
      clear.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            area.setText("");		
         }            
      });
      startStop.add(start);
      startStop.add(stop);
      size = new JTextField();
      size.setText("1000");
      iterations = new JTextField();
      iterations.setText("10000000");
      config.add(alg);
      JPanel tmp = new JPanel(new BorderLayout());
      tmp.add(new JLabel("Size: "), BorderLayout.WEST);
      tmp.add(size, BorderLayout.CENTER);
      config.add(tmp);
      tmp = new JPanel(new BorderLayout());
      tmp.add(new JLabel("Iterations: "), BorderLayout.WEST);
      tmp.add(iterations, BorderLayout.CENTER);
      config.add(tmp);
      config.add(clear);
      panel.add(config, BorderLayout.NORTH);
      panel.add(startStop, BorderLayout.SOUTH);
      frame.pack();
      frame.setVisible(true);
   }

   public void actionPerformed(ActionEvent e) {      
      if (e.getSource() == stop) {
         Thread t = this.thread;
         if (t != null) {
            t.stop();
            stop.setEnabled(false);
            area.append("\nTest stopped!");
            start.setEnabled(true);
         }
      } else if (e.getSource() == start) {
         String txt = this.size.getText();
         String txt2 = this.iterations.getText();
         try {
            final int size = Integer.parseInt(txt);
            final int iter = Integer.parseInt(txt2);
            if (size < 0 || iter < 0) {
               area.append("\nnegative Numbers not allowed");
               return;
            }
            start.setEnabled(false);
            thread = new Thread(new Runnable() {
               public void run() {
                  int[] a; int[] b;
                  try {
                     a = new int[size];
                     b = new int[iter];
                     Random r = new Random(seed);
                     final int range = 2*size;
                     area.append("\n\nGenerating search array of size "+size);
                     for (int i = 0; i<size; i++) {
                        a[i] = r.nextInt(range);
                     }
                     area.append("\nGenerating access array of size "+iter);
                     for (int i = 0; i<iter; i++) {
                        b[i] = r.nextInt(range);
                     }
                     Alg alg = Alg.values()[SearchGui.this.alg.getSelectedIndex()];
                     long init = 0; 
                     System.gc();
                     switch (alg) {
                     case Linear:
                        area.append("\nStarting search with "+alg+"-search...");
                        init = System.currentTimeMillis();
                        for (int i = 0; i < iter; i++) {
                           Search.linearSearch(a, b[i]);
                        }
                        break;
                     case ImprovedLinear:
                        area.append("\nStarting search with "+alg+"-search...");
                        init = System.currentTimeMillis();
                        for (int i = 0; i < iter; i++) {
                           Search.improvedLinearSearch(a, b[i]);
                        }
                        break;
                     case Binary:
                        area.append("\nSorting array");
                        Sorting.quickSort(a);
                        area.append("\nStarting search with "+alg+"-search...");
                        init = System.currentTimeMillis();                        
                        for (int i = 0; i < iter; i++) {
                           Search.binarySearch(a, b[i]);
                        }
                        break;
                     case Interpolation:
                        area.append("\nSorting array");
                        Sorting.quickSort(a);
                        area.append("\nStarting search with "+alg+"-search...");
                        init = System.currentTimeMillis();                        
                        for (int i = 0; i < iter; i++) {
                           Search.interpolationSearch(a, b[i]);
                        }
                        break;
                     default:
                        area.append("\nunknown algorithm");
                     return;
                     }
                     long end = System.currentTimeMillis();
                     area.append("done ("+(end - init)+"ms" /* + ", " + hits + " hits" */ + ")");
                     System.gc();
                  } catch (OutOfMemoryError oom) {
                     a = null;
                     b = null;
                     System.gc();
                     JOptionPane.showMessageDialog(frame,
                           "Out of memory", "Out of memory", JOptionPane.ERROR_MESSAGE);			
                  } finally {
                     stop.setEnabled(false);
                     thread = null;
                     start.setEnabled(true);
                  }
               } 
            });
            stop.setEnabled(true);
            thread.start();	    
         } catch (NumberFormatException ne) {
            area.append(txt + " is not a valid number");
         }
      }
   }

}
