public class Sorting {

   static void swap(int[] a, int i, int j) {
      int tmp = a[i];
      a[i] = a[j];
      a[j] = tmp;
   }

   static void insertionSort(int[] a) {
      for (int i=1; i<a.length; i++) {
         int ai = a[i];
         int j = i;
         while (j > 0) {
            if (a[j-1] > ai) {
               a[j] = a[j-1];               
            } else {
               break;
            }
            j--;
         }
         a[j] = ai;
      }
   }

   static void downHeap(int[] a, int i, int n) {
      while (true) {
         int l = 2*i + 1;
         int r = 2*i + 2;
         int max = l < n && a[l] > a[i] ? l : i;
         max = r < n && a[r] > a[max] ? r : max;
         if (max == i) {
            break;
         } else {
            swap(a, max, i);
            i = max;
         }
      }
   }

   static void heapSort(int[] a) {
      int n = a.length;
      for (int i = n/2 - 1; i >= 0; i--) {
         downHeap(a, i, n);
      }
      for (int i=n-1; i >= 1; i--) {
         swap(a,0,i);
         downHeap(a, 0, i);
      }
   }

   static void quickSort(int[] a) {
      if (a.length > 0) swap(a, 0, minimum(a));
      quickSort(a, 1, a.length - 1);
   }
   static void quickSort(int[] a, int l, int r) {
      if (l < r) {
         int p = pivot(a,l,r); // l <= p <= r
         int v = a[p];
         int i = l - 1;
         int j = r;
         swap(a,p,r);
         while (true) {
            do i++; while (a[i] < v); 
            do j--; while (a[j] > v); 
            if (i >= j) {
               break;
            }
            swap(a,i,j); 
         }
         swap(a,r,i);
         quickSort(a,l,i-1);
         quickSort(a,i+1,r);         
      }
   }

   static void radixSort(int[] a) {
      throw new RuntimeException("this method was removed");
   }
   static void bucketSort(int[] a,int[] b,int mask,int shift) {
      int[] cnt_ind = new int[mask+1]; 
      int n = a.length;
      // count
      for (int i=0; i<n; i++) {
         int key = (a[i] >>> shift) & mask;
         cnt_ind[key]++;
      }
      // compute start-indices
      cnt_ind[mask] = n - cnt_ind[mask];
      for (int j=mask-1; j >= 0; j--) {
         cnt_ind[j] = cnt_ind[j+1] - cnt_ind[j];
      }
      // sort
      for (int i=0; i<n; i++) {
         int key = (a[i] >>> shift) & mask;
         int index = cnt_ind[key];
         b[index] = a[i];
         cnt_ind[key] = index+1;
      }
   }
   
   static int pivotSelect(int[] a, int l, int r) {
      throw new RuntimeException("this method was removed");
   }
   static void quickSelect(int[] a, int x) {
      if (x < 0 || x >= a.length) {
         throw new RuntimeException("no x-th element");
      }
      swap(a, 0, minimum(a));
      if (x > 0) {
         quickSelect(a, 1, a.length-1, x);
      }
   }

   static void quickSelect(int[] a, int l, int r, int x) {
      while (l < r) {
         int p = pivotSelect(a,l,r); 
         int v = a[p]; // Partition: von hier bis ...
         int i = l - 1; 
         int j = r;
         swap(a,p,r);
         while (true) {
            do i++; while (a[i] < v); 
            do j--; while (a[j] > v); 
            if (i >= j) {
               break;
            }
            swap(a,i,j); 
         }
         swap(a,r,i); // ... hier gleich zu quickSort
         if (i > x) {
            r = i-1;
         } else if (i == x) {
            return;
         } else {
            l = i+1;
         }
      }
   }
   
   static void quickSortMedian(int[] a) {
      if (a.length > 0) swap(a, 0, minimum(a));
      quickSortMedian(a, 1, a.length - 1);
   }
   
   static void quickSortMedian(int[] a, int l, int r) {
      if (l < r) {
         int m = (l+1)/2 + r/2;
         quickSelect(a, l, r, m);
         quickSortMedian(a,l,m-1);
         quickSortMedian(a,m+1,r);         
      }
   }

   static enum Pivot {Right, Middle, Random, Median, LeftThird};
   static Pivot pivot = Pivot.Random;

   static int pivot(int[] a, int l, int r) {
      switch(pivot) {
      case Right: return r;
      case Middle: return (l+1)/2 + r/2;
      case LeftThird: return l + (r - l) / 3;
      case Random: return l + (int)(Math.random() * (r - l));
      case Median:
         int m = (l+1)/2 + r/2;
         quickSelect(a, l, r, m);
         return m;
      default: throw new RuntimeException("Unknown Pivot Function");
      }
   }

   static int radixBits = 8;


   // determines position of minimum element
   static int minimum(int[] a) {
      int n = a.length;      
      if (n > 0) {
         int min = a[0];
         int index = 0;
         for (int j=1; j<n; j++) {
            if (a[j] < min) {
               index = j;
               min = a[j];
            }
         }
         return index;
      } else {
         return -1;
      }
   }


   static boolean sorted(int[] a) {
      for (int i=1; i<a.length; i++) {
         if (a[i] < a[i-1]) {
            return false;
         }
      }
      return true;
   }   

   static int[] random(int n, int range) {
      int[] a = new int[n];
      for (int i=0; i<n; i++) {
         a[i] = (int) (range * Math.random());
      }
      return a;
   }

}
