import java.awt.*;
import java.awt.event.*;
import java.util.Random;

import javax.swing.*;

public class StackGui implements ActionListener {

   public static void main(String[] args) {
      new StackGui();
   }

   final JFrame frame;
   final JButton array, list;
   final JTextField nr;
   final JTextArea area;
   final long seed;

   StackGui() {
      seed = (long)(Math.random() * Long.MAX_VALUE); 
      frame = new JFrame("Stack-Test");
      frame.setPreferredSize(new Dimension(600,400));
      JPanel panel = new JPanel(new BorderLayout());
      frame.setContentPane(panel);
      area = new JLogArea();
      area.setText("Testing List- and Array-Stacks\n");
      JScrollPane scroll = new JScrollPane(area);
      panel.add(scroll, BorderLayout.CENTER);
      JPanel config = new JPanel(new GridLayout(1,4));
      array = new JButton("Array-Stack");
      list = new JButton("List-Stack");
      JButton clear = new JButton("Clear");
      array.addActionListener(this);
      list.addActionListener(this);
      clear.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            area.setText("");		
         }            
      });
      nr = new JTextField();
      nr.setText("20000");
      config.add(list);
      config.add(array);
      config.add(nr);
      config.add(clear);
      panel.add(config, BorderLayout.NORTH);
      frame.pack();
      frame.setVisible(true);
   }

   public void actionPerformed(ActionEvent e) {
      String txt = this.nr.getText();
      try {
         final int nr = Integer.parseInt(txt);
         if (nr < 0) {
            area.append("\nnegative Numbers not allowed");
            return;
         }
         final boolean list = e.getSource() == this.list;
         Thread t = new Thread(new Runnable() {
            public void run() {
               Stack s = list ? new ListStack() : new ArrayStack();
               try {
                  Random r = new Random(seed);
                  System.gc();
                  area.append("\n\nTesting " + (list ? "List" : "Array") + "-Stack");
                  area.append("\nPushing "+nr+" elements... ");
                  long init = System.currentTimeMillis();
                  long init2 = init;
                  for (int i = 0; i < nr; i++) {
                     s.push(i);
                  }
                  long mid = System.currentTimeMillis();
                  area.append("done ("+(mid - init2)+"ms)");
                  init2 = mid;
                  area.append("\nPopping "+nr+" elements... ");
                  for (int i = 0; i < nr; i++) {
                     s.pop();
                  }
                  mid = System.currentTimeMillis();
                  area.append("done ("+(mid - init2)+"ms)");
                  init2 = mid;
                  area.append("\nPushing "+nr+" elements... ");
                  for (int i = 0; i < nr; i++) {
                     s.push(i);
                  }
                  mid = System.currentTimeMillis();
                  area.append("done ("+(mid - init2)+"ms)");
                  init2 = mid;
                  area.append("\n"+nr+" random operations... ");
                  for (int i = 0; i < nr; i++) {
                     if (r.nextBoolean()) {
                        s.push(i);
                     } else {
                        s.pop();
                     }
                  }
                  mid = System.currentTimeMillis();
                  area.append("done ("+(mid - init2)+"ms)");
                  init2 = mid;
                  area.append("\ntotal time: "+ (mid - init) + "ms");
                  // JOptionPane.showMessageDialog(frame, "Total time: " + (mid - init), "Tests finished", JOptionPane.INFORMATION_MESSAGE);
                  System.gc();
               } catch (OutOfMemoryError oom) {
                  s = null;
                  System.gc();
                  JOptionPane.showMessageDialog(frame,
                        "Out of memory", "Out of memory", JOptionPane.ERROR_MESSAGE);			
               }
            } 
         });
         t.start();	    
      } catch (NumberFormatException ne) {
         area.append(txt + " is not a valid number");
      }
   }

}
